;+======================================================+;
;|          CALCnet2.2 Speed Burst Testing              |;
;|                  SpeedTest 1.1                       |;
;|                 by Kerm Martian                      |;
;|       (c) 2002-2010 Cemetech and Kerm Martian        |;
;|             http://www.cemetech.net                  |;
;+======================================================+;
; All rights reserved.  Reuse or republication of this   ;
; code in whole or in part in any form, textual, binary, ;
; electronic, or otherwise without express written       ;
; permission from Christopher "Kerm Martian" Mitchell is ;
; strictly prohibited.                                   ;
;+======================================================+;

;===Standard Includes and Equates===
.nolist
#define TASM
#include "ti83plus.inc"
#include "dcs7.inc"

#define unidir						;unidirectional test
;#define bidir						;bidirectional test

totalbytes_sent			.equ		saferam2		;2 bytes
totalbytes_recv 		.equ		saferam2+2		;2 bytes
totalbytes_sent_inchead	.equ		saferam2+4		;2 bytes
totalbytes_recv_inchead .equ		saferam2+6		;2 bytes
totalframes_recv		.equ		saferam2+8		;2 bytes
prgm_mode				.equ		saferam2+10		;1 byte
othercalc_id			.equ		saferam2+11		;5 bytes
saferam_size			.equ		othercalc_id+5-saferam2

#define FRAME_DATA_SIZE 90
.list
	.org progstart
#IFDEF TI83P
	.db $BB,$6D
#ENDIF
INIT:
	xor d
#IFDEF TI83P
	.db $C9
#ENDIF
	jr Start
	.dw Description
	.db $07,$01
	.dw Icon
	.dw $0000
	.dw $0000			;the routine to open files.  DCS will start you here instead of at $9D95
						;if a file is pending
Description:
	.db "Cn2.2 Speed Burst Test",0
Icon:				;a 16x16 icon
	.db %00110000,%00000000
	.db %01111000,%00000110
	.db %11011000,%00001011
	.db %11000000,%00000011
	.db %11000000,%00000110
	.db %11000000,%00001100
	.db %11000000,%00001111
	.db %11000010,%11100000
	.db %11000011,%10110000
	.db %11000011,%00110000
	.db %11000011,%00110000
	.db %11000011,%00110000
	.db %11000011,%00110000
	.db %11011011,%00110000
	.db %01111011,%00110000
	.db %00110011,%00110000

START:                          	;main routines
	set textwrite,(iy+sgrflags)		;set the textwrite flag
	ld hl,saferam2
	ld bc,saferam_size-1
	ld (hl),0
	push hl
		pop de
	inc de
	ldir							;clear out the saferam
	
Start_DBLoop:						;debounce...
	ld b,16
Start_DBInnerLoop:
	call Cn2_GetK
	or a
	jr nz,Start_DBLoop
	djnz Start_DBInnerLoop

	call Cn2_Setup					;set up calcnet 2.2

Start_FindCalcsRedraw:
	call QuickLCDClear
	ld ix,stsplash
	ld b,37
	ld c,72/8
	ld a,12
	ld l,0
	call iLargeSprite

	ld hl,(38*256)+1
	ld (pencol),hl
	ld hl,PressForMode0
	bcall(_vputs)
	ld hl,(44*256)+1
	ld (pencol),hl
	ld hl,PressForMode1
	bcall(_vputs)
	ld hl,(56*256)+1
	ld (pencol),hl
	ld hl,MyName
	bcall(_vputs)
Start_FindCalcsOuter:
	call iFastCopy
	call Cn2_GetK
	cp skClear
	jp z,FinalExit
	cp sk1
	jp z,Start_SetCalcMode1
	cp sk2
	jp z,Start_SetCalcMode2
Start_FindCalcsRandPause:
	ld b,254
	call iRandom
	ld b,a
Start_FindCalcRandWait:
	push hl \ pop hl
	push hl \ pop hl
	djnz Start_FindCalcRandWait
	ld a,(Cn2_Int_RecBuf+6)
	or a
	jr nz,Start_FindCalcs_BCastReqRecvd
	ld a,(prgm_mode)
	or a
	jr z,Start_FindCalcsOuter
	ld a,(Cn2_Int_SendBuf+6)
	or a
	jr nz,Start_FindCalcsOuter
	ld b,20
	call iRandom
	or a
	jr nz,Start_FindCalcsOuter
Start_FindCalcs_SendBcast:
	ld hl,FindCalcsMsg
	ld de,Cn2_Int_SendBuf
	ld bc,FindCalcsMsgEnd-FindCalcsMsg
	ldir
	dec de
	ld a,(prgm_mode)
	add a,2									;mode 1=3, mode 2=4
	ld (de),a								;set mode 1 or 2
	ld hl,Cn2_Int_SendBuf+6
	ld (hl),$80
	jp Start_FindCalcsOuter
Start_FindCalcs_BCastReqRecvd:
	ld hl,Cn2_Int_RecBuf+6
	ld (hl),0
	inc hl
	ld a,(hl)
	cp $03									;this calc will be the transmitter for mode 1	(unidir)
	jp z,Start_FindCalcs_BCastReqRecvd_Mode1
	cp $04									;this calc will be the first sender for mode 2 (bidir)
	jp z,Start_FindCalcs_BCastReqRecvd_Mode2
	cp $05									;Packet type 5 is the data chunk!
	jp nz,FinalExit							;unknown?!?! die immediately.
Start_FindCalcs_DataFrameRecvd:
	ld hl,Cn2_Int_RecBuf+6
	xor a
	ld (hl),a
	dec hl
	ld l,(hl)
	ld h,a
	push hl
		ld de,(totalbytes_recv)
		add hl,de
		ld (totalbytes_recv),hl
		pop hl
	ld de,10								;5 ID + 2 size + 2 checksum + 1 ack
	add hl,de
	ld de,(totalbytes_recv_inchead)
	add hl,de
	ld (totalbytes_recv_inchead),hl
;	xor a
;	ld (Cn2_Int_RecBuf+6),a					;allow a new frame to be received
	ld hl,Cn2_Int_SendBuf+6
	ld (hl),0								;clear out anything pending to send, like repeated broadcasts, for instance
	ld a,(prgm_mode)
	cp 1
	jr nz,Start_FindCalcs_DataFrameRecvd_SetReply
	call DisplayStats
	jr MainWaitKey
Start_FindCalcs_DataFrameRecvd_SetReply:
	;ld hl,othercalc_id
	ld hl,Cn2_Int_RecBuf
;Start_FindCalcs_DataFrameRecvd_SetReply_HLExists:
	ld de,Cn2_Int_SendBuf
	ld bc,5
	ldir
	ex de,hl
	;inc de					;cn2_int_sendbuf+6
	ld (hl),FRAME_DATA_SIZE
	inc hl					;cn2_int_sendbuf+6
	ld (hl),0
	inc hl
	ld (hl),5
	dec hl
	ld (hl),$80						;cn2_int_sendbuf+6
	ld de,FRAME_DATA_SIZE
	ld hl,(totalbytes_sent)
	add hl,de
	ld (totalbytes_sent),hl
	ld de,FRAME_DATA_SIZE+10
	ld hl,(totalbytes_sent_inchead)
	add hl,de
	ld (totalbytes_sent_inchead),hl
	call DisplayStats	
MainWaitKey:
	call iFastCopy
	call Cn2_GetK
	cp skClear
	jp z,FinalExit
	ld a,(Cn2_Int_RecBuf+6)
	or a
	jp nz,Start_FindCalcs_BCastReqRecvd
	ld a,(prgm_mode)
	cp 3
	jr nz,MainWaitKey
	ld a,(Cn2_Int_SendBuf+6)
	or a
	jp z,Start_FindCalcs_DataFrameRecvd_SetReply
	jr MainWaitKey
	
Start_FindCalcs_BCastReqRecvd_Mode1:
Start_FindCalcs_BCastReqRecvd_Mode2:
	ld (prgm_mode),a								;a is already 1 or 2 (actually, 3 or 4)
	ld hl,Cn2_Int_RecBuf
;	ld de,othercalc_id
;	ld bc,5
;	ldir
	jr Start_FindCalcs_DataFrameRecvd_SetReply

Start_SetCalcMode1:
	ld a,1
	jr Start_SendBCast
Start_SetCalcMode2:
	ld a,2
Start_SendBCast:
	ld (prgm_mode),a
	jp Start_FindCalcsOuter
	
	
DisplayStats:
	call iCheckInts
	push af
		di
		ld hl,(38*256)+1
		ld (pencol),hl
		ld hl,SentTxt
		bcall(_vputs)
		ld hl,(totalbytes_sent)
		call vdisphl
		ld a,'/'
		bcall(_vputmap)
		ld hl,(totalbytes_sent_inchead)
		call vdisphl
		ld hl,(44*256)+1
		ld (pencol),hl
		ld hl,RcvdTxt
		bcall(_vputs)
		ld hl,(totalbytes_recv)
		call vdisphl
		ld a,'/'
		bcall(_vputmap)
		ld hl,(totalbytes_recv_inchead)
		call vdisphl
		pop af
	ret po
	ei
	ret

QuickLCDClear:
	ld hl,gbuf
	push hl
		pop de
	inc de
	ld (hl),0
	ld bc,767
	ldir							;clear the gbuf
	ret
	
Cn2GetK_Sub:
	ld a,$ff		;
	out (1),a		;reset keyport
	ld e,$fe		;frist group
	ld c,$01		;key port
	ld l,0		;l holds key pressed
cscloop:
	out (c),e		;set keygroup
	ld b,8		;loop, Delay needed when work with key driver
	nop \ nop
	in a,(c)		;read key
cscbit:
	inc l			;inc to get key pressed
	rra 			; if key pressed done
	jr nc,donecsc
	djnz cscbit 	;loop 8
	rlc e			;next key group
	jp m,cscloop	;if bit 7 set loop
	ld l,0		;if no key pressed 0
donecsc:
	ld a,l		;
	or a
	ret

FinalExit:
	call Cn2_Setdown
	ret

iCheckInts:
;Push a zero byte to the stack and pop it
	xor a
	push af
		pop af
	ld a,i
	ret pe
	;See if an interrupt triggered. If so, the byte on the stack will not be 0 anymore
	dec sp
	dec sp
	pop af
	add a,a
	ret z
	xor a
	ret

PressForMode0:
	.db "Press num: ",$c1,"1]=Uni sender",0
PressForMode1:
	.db $c1,"2]=Bidir endpoint",0
MyName:
	.db "Kerm Martian & Cemetech",0
SentTxt:
	.db "Sent: ",0
RcvdTxt:
	.db "Rcvd: ",0

FindCalcsMsg:
	.db $00,$00,$00,$00,$00
	.dw $0001
	.db 1
FindCalcsMsgEnd:

stsplash:
	;---------------------Generated by SourceCoder 2.5, http://sc.cemetech.net-------------------
	.db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
	.db %00000000,%00001111,%10000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
	.db %00000000,%00111111,%10000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
	.db %00000000,%11110000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
	.db %00000011,%11000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000111,%10001111
	.db %00000111,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%10000001
	.db %00001100,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%11000000,%10000001
	.db %00011000,%00111111,%00111110,%01100000,%01111110,%00000000,%00000000,%11000111,%10001111
	.db %00110000,%01111111,%01111111,%01100000,%11111110,%00000000,%00000000,%11000100,%00001000
	.db %00110000,%01100000,%01100011,%01100000,%11000000,%00000000,%00000000,%11110100,%00001000
	.db %01100000,%01100000,%01100011,%01100000,%11000000,%11111100,%01111100,%11110111,%10101111
	.db %01100000,%01100000,%01100011,%01100000,%11000000,%11111110,%11111110,%11000000,%00000000
	.db %01100000,%01100000,%01100011,%01100000,%11000000,%11000110,%11000110,%11000000,%00000000
	.db %01100000,%01100000,%01111111,%01100000,%11000000,%11000110,%11111110,%11000000,%00000000
	.db %01100000,%01100000,%01111111,%01100000,%11000000,%11000110,%11111110,%11000000,%00000000
	.db %01100000,%01100000,%01100011,%01100000,%11000000,%11000110,%11000000,%11000000,%00000000
	.db %01100000,%01100000,%01100011,%01100000,%11000000,%11000110,%11000000,%11000000,%00000000
	.db %00110000,%01100000,%01100011,%01100000,%11000000,%11000110,%11000000,%11000000,%10000000
	.db %00110000,%01111111,%01100011,%01111110,%11111110,%11000110,%11111110,%11110001,%11000000
	.db %00011000,%00111111,%01100011,%01111110,%01111110,%11000110,%01111110,%01110000,%11100000
	.db %00001100,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%01110000
	.db %00000110,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00111000
	.db %00000011,%10000000,%00000000,%00000000,%00000000,%00000011,%11111111,%11111111,%11111100
	.db %00000001,%11100000,%00000000,%00000000,%00111111,%11111111,%11111111,%11111111,%11111100
	.db %00000000,%01111100,%00000001,%11111111,%11111111,%11111100,%00000000,%00000000,%00111000
	.db %00000000,%00011111,%11111111,%11111111,%11000000,%00000000,%00000000,%00000000,%01110000
	.db %00000000,%00000011,%11111110,%00000000,%00000000,%00000000,%00000000,%00000000,%11100000
	.db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001,%11000000
	.db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001,%10000000
	.db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
	.db %11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111
	.db %11111110,%00010000,%11000001,%00000100,%00111111,%00000100,%00011000,%01000001,%11111111
	.db %11111101,%11110111,%01011111,%01111101,%11011111,%11011101,%11110111,%11110111,%11111111
	.db %11111110,%00110000,%11000111,%00011101,%11011111,%11011100,%01111000,%11110111,%11111111
	.db %11111111,%11010111,%11011111,%01111101,%11011111,%11011101,%11111111,%01110111,%11111111
	.db %11111100,%00110111,%11000001,%00000100,%00111111,%11011100,%00010000,%11110111,%11111111
	.db %11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111

;#include "../source/calcnet2.asm"

.end
END
